import React, { useMemo } from 'react';

import { AccordionItem, NsAccordionItem, NsAccordionProps, useAccordionMap } from '..';
import { StyledAccordionWrapper } from './ToggleSection.styled';

/**
 * NsAccordion can be used to show a list of items with title and body
 * it accepts a `boolean` prop `allowMultipleExpanded` which when true,
 * multiple sections of accordion can remain open at a time
 *
 * we can define certain section to remain open by default using `isDefaultOpen` in the
 * items
 * if there are multiple items available for default open, but `allowMultipleExpanded`
 * is false then only first item of default open is made open
 */
const createKey = (item: AccordionItem, index: number) => item?.id + index;
const NsAccordion = ({
    allowMultipleExpanded,
    invert,
    level,
    variant,
    items,
    headerTypographyVariant,
    closedIconName,
    openedIconName,
    additionalProps,
    isChild = false,
}: NsAccordionProps) => {
    const defaultOpenIds = useMemo(() => {
        return items
            ?.map?.((content, index) => (content?.isDefaultOpen ? createKey(content, index) : null))
            .filter((e) => e !== null) as string[];
    }, [items]);

    const { toggleOpen, itemStateMap } = useAccordionMap({ allowMultipleExpanded, defaultOpenIds });

    return (
        <StyledAccordionWrapper
            variant={variant}
            className="ns-accordion"
            data-testid={additionalProps?.testid ?? 'ns-accordion-testid'}
        >
            {items?.map?.((item, index) => {
                const key = createKey(item, index);
                const isOpen = itemStateMap[key] ?? false;
                return (
                    <NsAccordionItem
                        allowMultipleExpanded={item.allowMultipleExpanded}
                        type={item.type}
                        closedIconName={closedIconName}
                        content={item.content}
                        headerTypographyVariant={headerTypographyVariant}
                        invert={invert}
                        isOpen={isOpen}
                        isChild={isChild}
                        key={key}
                        level={level}
                        openedIconName={openedIconName}
                        startIcon={item.icon}
                        toggleOpen={() => toggleOpen(createKey(item, index))}
                        title={<div className="nsa-item-title">{item.title}</div>}
                        variant={variant}
                    />
                );
            })}
        </StyledAccordionWrapper>
    );
};

export default NsAccordion;
