import React from 'react';
import { render, fireEvent, screen } from '@/test-utils/index';
import { AccordionContentTypeObject, AccordionItem, AccordionVariant, NsAccordion } from '@/common/index';

const generateContentArray = (count: number, isDefaultOpen = true) => {
    const arr = [] as AccordionItem[];
    for (let i = 1; i <= count; i++) {
        arr?.push({
            title: `Title ${i}`,
            content: <div className="p-2">Content {i}</div>,
            isDefaultOpen,
            type: AccordionContentTypeObject.jsx,
            id: `Test ${i}`,
        });
    }
    return arr;
};

describe('NsAccordion', () => {
    const items = generateContentArray(3);
    const defaultProps = {
        additionalProps: {
            testid: 'test-accordion',
        },
        allowMultipleExpanded: false,
        items,
        variant: AccordionVariant.Basic,
        invert: false,
        level: 0,
    };

    it('1. renders without error', () => {
        const { getByTestId } = render(<NsAccordion {...defaultProps} />);
        const accordion = getByTestId(defaultProps.additionalProps.testid);
        expect(accordion).toBeInTheDocument();
    });

    it(`2. only first section should be visible when allowMultipleExpanded prop is false and multiple items have
    isDefaultOpen true`, () => {
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'false');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
        const [c1, c2, c3] = getAllByText(/Content /i);
        expect(c1.parentNode).toHaveStyle('opacity: 1');
        expect(c2.parentNode).toHaveStyle('opacity: 0');
        expect(c3.parentNode).toHaveStyle('opacity: 0');
    });

    it(`3. expands second section only when allowMultipleExpanded
     prop is false and second title is clicked`, async () => {
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);
        const header = title2.closest('header');
        expect(header).toHaveAttribute('aria-expanded', 'false');
        if (header) {
            fireEvent.click(header);
            expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
            expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
            const [c1, c2, c3] = getAllByText(/Content /i);
            expect(c1.parentNode).toHaveStyle('opacity: 0');
            expect(c2.parentNode).toHaveStyle('opacity: 1');
            expect(c3.parentNode).toHaveStyle('opacity: 0');
        }
    });

    it('4. expands multiple sections by default when allowMultipleExpanded prop is true', () => {
        const items = generateContentArray(3);
        items[2].isDefaultOpen = false;
        const defaultProps = {
            allowMultipleExpanded: false,
            items,
            variant: AccordionVariant.Basic,
            level: 0,
            invert: false,
        };
        const { getAllByText } = render(<NsAccordion {...defaultProps} allowMultipleExpanded />);
        const accordion = screen.getByTestId('ns-accordion-testid');
        expect(accordion).toBeDefined();
        const [title1, title2, title3] = getAllByText(/Title /i);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');

        const [c1, c2, c3] = getAllByText(/Content /i);
        expect(c1.parentNode).toHaveStyle('opacity: 1');
        expect(c2.parentNode).toHaveStyle('opacity: 1');
        expect(c3.parentNode).toHaveStyle('opacity: 0');

        fireEvent.click(title3);

        expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'true');
        expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'true');

        const [c11, c12, c13] = getAllByText(/Content /i);
        expect(c11.parentNode).toHaveStyle('opacity: 1');
        expect(c12.parentNode).toHaveStyle('opacity: 1');
        expect(c13.parentNode).toHaveStyle('opacity: 1');
    });
    it(`5. closes second section also when allowMultipleExpanded
     prop is true and second title is clicked`, async () => {
        const items = generateContentArray(3);
        const defaultProps = {
            additionalProps: {
                testid: 'test-accordion',
            },
            allowMultipleExpanded: true,
            items,
            variant: AccordionVariant.Basic,
            invert: false,
            level: 0,
        };
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);
        const header = title2.closest('header');
        expect(header).toHaveAttribute('aria-expanded', 'true');
        if (header) {
            fireEvent.click(header);
            expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'true');
            expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'true');

            const [c1, c2, c3] = getAllByText(/Content /i);
            expect(c1.parentNode).toHaveStyle('opacity: 1');
            expect(c2.parentNode).toHaveStyle('opacity: 0');
            expect(c3.parentNode).toHaveStyle('opacity: 1');
        }
    });
    it(`6. closes second section also when allowMultipleExpanded
     prop is true and second title is clicked`, async () => {
        const items = generateContentArray(3, false);
        items[1].isDefaultOpen = true;
        const defaultProps = {
            additionalProps: {
                testid: 'test-accordion',
            },
            allowMultipleExpanded: false,
            items,
            variant: AccordionVariant.Compact,
            level: 0,
            invert: false,
        };
        const { getAllByText } = render(<NsAccordion {...defaultProps} />);
        const [title1, title2, title3] = getAllByText(/Title /i);
        const header = title2.closest('header');
        expect(header).toHaveAttribute('aria-expanded', 'true');
        if (header) {
            fireEvent.click(header);
            expect(title1.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(title2.closest('header')).toHaveAttribute('aria-expanded', 'false');
            expect(title3.closest('header')).toHaveAttribute('aria-expanded', 'false');
            const [c1, c2, c3] = getAllByText(/Content /i);
            expect(c1.parentNode).toHaveStyle('opacity: 0');
            expect(c2.parentNode).toHaveStyle('opacity: 0');
            expect(c3.parentNode).toHaveStyle('opacity: 0');
        }
    });
});
