import React from 'react';
import { render, fireEvent, screen, waitFor } from '@/test-utils/index';
import NsDropdown from './NsDropdown';
import NsDropdownOption from './NsDropdownOption';

function createDropdownOptions(options: { value: string; label: string }[]) {
    return options.map((option) => <NsDropdownOption key={option.value} value={option.value} label={option.label} />);
}

describe('NsDropdown', () => {
    const onSelect = jest.fn();
    const selectedValue = '';
    const selectedLabel = '';

    const options = [
        { value: 'Option 1', label: 'Option 1' },
        { value: 'Option 2', label: 'Option 2' },
        { value: 'Option 3', label: 'Option 3' },
    ];

    it('renders the placeholder text when no value is selected', () => {
        render(
            <NsDropdown
                placeholder="Select an option"
                selectedValue={selectedValue}
                selectedLabel={selectedLabel}
                onSelect={onSelect}
            >
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        expect(screen.getByText('Select an option')).toBeInTheDocument();
    });

    it('renders the selected value label', () => {
        const selectedLabel = 'Option 1';
        render(
            <NsDropdown selectedValue="Option 1" selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        expect(screen.getByText(selectedLabel)).toBeInTheDocument();
    });

    it('dropdown opens and closes on click', () => {
        render(
            <NsDropdown selectedValue={selectedValue} selectedLabel={selectedLabel} onSelect={onSelect}>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        expect(screen.getByTestId('option-Option 1')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 2')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 3')).toBeInTheDocument();

        fireEvent.click(dropdownHead);
        expect(screen.queryByTestId('option-Option 1')).not.toBeInTheDocument();
        expect(screen.queryByTestId('option-Option 2')).not.toBeInTheDocument();
        expect(screen.queryByTestId('option-Option 3')).not.toBeInTheDocument();
    });

    it('dropdown closes on mouse leave when closeOnMouseLeave is set to true', async () => {
        render(
            <NsDropdown
                closeOnMouseLeave
                selectedValue={selectedValue}
                selectedLabel={selectedLabel}
                onSelect={onSelect}
            >
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        const dropdownList = screen.getByTestId('dropdown-list');
        fireEvent.mouseLeave(dropdownList);

        await waitFor(() => {
            expect(screen.queryByTestId('option-Option 1')).not.toBeInTheDocument();
            expect(screen.queryByTestId('option-Option 2')).not.toBeInTheDocument();
            expect(screen.queryByTestId('option-Option 3')).not.toBeInTheDocument();
        });
    });
    it('dropdown does not close on mouse leave when closeOnMouseLeave is set to false', () => {
        render(
            <NsDropdown
                selectedValue={selectedValue}
                selectedLabel={selectedLabel}
                onSelect={onSelect}
                closeOnMouseLeave={false}
            >
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        const dropdownList = screen.getByTestId('dropdown-list');
        fireEvent.mouseLeave(dropdownList);

        expect(screen.getByTestId('option-Option 1')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 2')).toBeInTheDocument();
        expect(screen.getByTestId('option-Option 3')).toBeInTheDocument();
    });
    it('dropdown does not open when disabled', () => {
        render(
            <NsDropdown selectedValue={selectedValue} selectedLabel={selectedLabel} onSelect={onSelect} disabled>
                {createDropdownOptions(options)}
            </NsDropdown>
        );

        const dropdownHead = screen.getByTestId('dropdown-head');
        fireEvent.click(dropdownHead);

        expect(screen.queryByTestId('dropdown-list')).not.toBeInTheDocument();
    });
});
